/*******************************************************************************
 * Copyright (c) 2000, 2006 Eteration A.S. and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Eteration A.S. - initial API and implementation
 *******************************************************************************/
package org.eclipse.wtp.releng.monitor;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.apache.tools.ant.Task;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class BuildMonitor extends Task {

	public BuildMonitor() {
		testResultsWithProblems = "\n";
		parser = null;
		testResultsTemplateString = "";
		dropTemplateString = "";
		testsRan = true;
	}

	public static void main(String args[]) {
		BuildMonitor test = new BuildMonitor();
		test.setDropTokenList("%wst%,%jst%,%wstsdk%,%jstsdk%");
		test.getDropTokensFromList(test.dropTokenList);
		test.setIsBuildTested(true);
		test
				.setXmlDirectoryName("C:/nmd/dev/projects/wtpr1/org.eclipse.wtp.releng.monitor/test-build/testResults/xml");
		test
				.setTestResultsTemplateFileName("C:\\nmd\\dev\\projects\\wtpr1\\org.eclipse.wtp.releng\\templateFiles\\testResults.php.template");
		test
				.setCompileLogsDirectoryName("C:/nmd/dev/projects/wtpr1/org.eclipse.wtp.releng.monitor/test-build/compilelogs");
		test
				.setTestManifestFileName("C:\\nmd\\dev\\projects\\wtpr1\\org.eclipse.wtp.releng\\testManifest.xml");
		test.setHostname("crm.lomboz.org");
		test.setSender("ndai@lomboz.org");
		test.setPrefix("[wtp-build]");
		test.setBuildLabel("I-I20050414-200504142052");
		test
				.setBuildLink("http://download.eclipse.org/webtools/downloads/drops/I-I20050414-200504142052");
		test.setRecipients("ndai@lomboz.org");
		test.execute();
	}

	public void execute() {
		anErrorTracker = new ErrorTracker();
		anErrorTracker.loadFile(testManifestFileName);
		getDropTokensFromList(dropTokenList);
		System.out.println("Begin: Reading test results template");
		testResultsTemplateString = readFile(testResultsTemplateFileName);
		System.out.println("Parsing test results XML files");
		parseXml();
		System.out.println("Parsing compile logs");
		parseCompileLogs();
		System.out.println("Sending notice");
		sendNotice();
		System.out.println("End: monitor");
	}

	public void parseCompileLogs() {
		String replaceString = "";
		replaceString = processCompileLogsDirectory(compileLogsDirectoryName,
				replaceString);
		if (replaceString == "")
			replaceString = "None";
		testResultsTemplateString = replace(testResultsTemplateString,
				"%compilelogs%", replaceString);
	}

	private String processCompileLogsDirectory(String directoryName,
			String aString) {
		File sourceDirectory = new File(directoryName);
		String replaceString = aString;
		if (sourceDirectory.exists()) {
			File directories[] = sourceDirectory.listFiles();
			if (directories == null)
				return "There are no compile logs";
			Arrays.sort(directories);
			for (int i = 0; i < directories.length; i++)
				if (directories[i].isDirectory()) {
					File logFiles[] = directories[i].listFiles();
					Arrays.sort(logFiles);
					for (int j = 0; j < logFiles.length; j++) {
						String longName = logFiles[j].getPath();
						if (logFiles[j].isDirectory())
							replaceString = replaceString
									+ processCompileLogsDirectory(longName,
											aString);
						else if (longName.endsWith(".log"))
							replaceString = replaceString
									+ readCompileLog(longName);
					}

				} else {
					replaceString = replaceString
							+ readCompileLog(directories[i].getAbsolutePath());
				}

		} else {
			System.out.println("Test results not found in "
					+ sourceDirectory.getAbsolutePath());
		}
		return replaceString;
	}

	private String readCompileLog(String log) {
		String fileContents = readFile(log);
		int errorCount = countCompileErrors(fileContents);
		int warningCount = countCompileWarnings(fileContents);
		if (errorCount != 0) {
			String logName = log.substring(getCompileLogsDirectoryName()
					.length() + 1);
			StringBuffer buffer = new StringBuffer(logName);
			buffer.replace(logName.indexOf("_") + 1, logName.indexOf(
					File.separator, logName.indexOf("_") + 1), "*");
			logName = new String(buffer);
			anErrorTracker.registerError(logName);
		}
		return formatCompileErrorRow(log, errorCount, warningCount);
	}

	public String readFile(String fileName) {
		try {
			FileInputStream aStream = new FileInputStream(fileName);
			byte aByteArray[] = new byte[aStream.available()];
			aStream.read(aByteArray);
			aStream.close();
			return new String(aByteArray);
		} catch (FileNotFoundException e) {
			System.out.println("File not found: " + fileName);
			return "";
		} catch (IOException e) {
			System.out.println("IOException: " + fileName);
		}
		return "";
	}

	private int countCompileErrors(String aString) {
		return extractNumber(aString, "error");
	}

	private int countCompileWarnings(String aString) {
		return extractNumber(aString, "warning");
	}

	private int extractNumber(String aString, String endToken) {
		int endIndex = aString.lastIndexOf(endToken);
		if (endIndex == -1)
			return 0;
		int startIndex;
		for (startIndex = endIndex; startIndex >= 0
				&& aString.charAt(startIndex) != '('
				&& aString.charAt(startIndex) != ','; startIndex--)
			;
		String count = aString.substring(startIndex + 1, endIndex).trim();
		try {
			return Integer.parseInt(count);
		} catch (NumberFormatException e) {
			return 0;
		}
	}

	private String verifyAllTestsRan(String directory) {
		Enumeration enumeration = anErrorTracker.getTestLogs().elements();
		String replaceString = "";
		while (enumeration.hasMoreElements()) {
			String testLogName = enumeration.nextElement().toString();
			if (!(new File(directory + File.separator + testLogName)).exists()) {
				anErrorTracker.registerError(testLogName);
				replaceString = replaceString
						+ formatRow(testLogName, -1, false);
				testResultsWithProblems = testResultsWithProblems.concat("\n"
						+ testLogName.substring(0, testLogName.length() - 4)
						+ " (file missing)");
			}
		}
		return replaceString;
	}

	public void parseXml() {
		File sourceDirectory = new File(xmlDirectoryName);
		if (sourceDirectory.exists()) {
			String replaceString = "";
			File xmlFileNames[] = sourceDirectory.listFiles();
			Arrays.sort(xmlFileNames);
			for (int i = 0; i < xmlFileNames.length; i++)
				if (xmlFileNames[i].getPath().endsWith(".xml")) {
					String fullName = xmlFileNames[i].getPath();
					int errorCount = countErrors(fullName);
					if (errorCount != 0) {
						String testName = xmlFileNames[i].getName().substring(
								0, xmlFileNames[i].getName().length() - 4);
						testResultsWithProblems = testResultsWithProblems
								.concat("\n" + testName);
						anErrorTracker.registerError(fullName
								.substring(getXmlDirectoryName().length() + 1));
					}
					replaceString = replaceString
							+ formatRow(xmlFileNames[i].getPath(), errorCount,
									true);
				}

			replaceString = replaceString + verifyAllTestsRan(xmlDirectoryName);
			testResultsTemplateString = replace(testResultsTemplateString,
					"%testresults%", replaceString);
			testsRan = true;
		} else {
			testsRan = false;
			System.out.println("Test results not found in "
					+ sourceDirectory.getAbsolutePath());
		}
	}

	private String replace(String source, String original, String replacement) {
		int replaceIndex = source.indexOf(original);
		if (replaceIndex > -1) {
			String resultString = source.substring(0, replaceIndex);
			resultString = resultString + replacement;
			resultString = resultString
					+ source.substring(replaceIndex + original.length());
			return resultString;
		} else {
			System.out.println("Could not find token: " + original);
			return source;
		}
	}

	public void sendNotice() {
		WtpMailer myMailer = new WtpMailer(getHostname(), getSender(),
				getRecipients(), getPrefix(), getBuildLabel(), getBuildLink());
		myMailer.sendMessage("build results", testResultsTemplateString);
	}

	public void setTestResultsTemplateFileName(String aString) {
		testResultsTemplateFileName = aString;
	}

	public String getTestResultsTemplateFileName() {
		return testResultsTemplateFileName;
	}

	public void setXmlDirectoryName(String aString) {
		xmlDirectoryName = aString;
	}

	public String getXmlDirectoryName() {
		return xmlDirectoryName;
	}

	private String formatCompileErrorRow(String fileName, int errorCount,
			int warningCount) {
		String aString = "";
		if (errorCount == 0 && warningCount == 0) {
			return aString;
		} else {
			int i = fileName.indexOf("compilelogs");
			String shortName = fileName.substring(i + "compilelogs".length());
			aString = aString + "<tr><td>" + shortName + "</td><td>";
			aString = aString + errorCount;
			aString = aString + "</td><td>";
			aString = aString + warningCount;
			aString = aString + "</td></tr>";
			return aString;
		}
	}

	private String formatRow(String fileName, int errorCount, boolean link) {
		String aString = "";
		if (!link)
			return "<tr><td>" + fileName + " (missing)" + "</td><td>" + "DNF";
		if (fileName.endsWith(".xml")) {
			int begin = fileName.lastIndexOf(File.separatorChar);
			int end = fileName.lastIndexOf(".xml");
			String shortName = fileName.substring(begin + 1, end);
			String displayName = shortName;
			if (errorCount != 0)
				aString = aString + "<tr><td><b>";
			else
				aString = aString + "<tr><td>";
			if (errorCount != 0)
				displayName = "<font color=\"#ff0000\">" + displayName
						+ "</font>";
			if (errorCount == -1)
				aString = aString.concat(displayName);
			else
				aString = aString + "<a href=" + "\"" + "testresults" + "/"
						+ shortName + ".html" + "\">" + displayName + "</a>";
			if (errorCount > 0)
				aString = aString + "</td><td><b>";
			else
				aString = aString + "</td><td>";
			if (errorCount == -1)
				aString = aString + "<font color=\"#ff0000\">DNF";
			else if (errorCount > 0)
				aString = aString + "<font color=\"#ff0000\">"
						+ String.valueOf(errorCount);
			else
				aString = aString + String.valueOf(errorCount);
			if (errorCount != 0)
				aString = aString + "</font></b></td></tr>";
			else
				aString = aString + "</td></tr>";
		}
		return aString;
	}

	private int countErrors(String fileName) {
		int errorCount = 0;
		if ((new File(fileName)).length() == 0L)
			return -1;
		try {
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
					.newInstance();
			parser = docBuilderFactory.newDocumentBuilder();
			Document document = parser.parse(fileName);
			NodeList elements = document.getElementsByTagName("testsuite");
			int elementCount = elements.getLength();
			if (elementCount == 0)
				return -1;
			for (int i = 0; i < elementCount; i++) {
				Element element = (Element) elements.item(i);
				NamedNodeMap attributes = element.getAttributes();
				Node aNode = attributes.getNamedItem("errors");
				errorCount += Integer.parseInt(aNode.getNodeValue());
				aNode = attributes.getNamedItem("failures");
				errorCount += Integer.parseInt(aNode.getNodeValue());
			}

		} catch (IOException e) {
			System.out.println("IOException: " + fileName);
			return 0;
		} catch (SAXException e) {
			System.out.println("SAXException: " + fileName);
			return 0;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return errorCount;
	}

	public String getCompileLogsDirectoryName() {
		return compileLogsDirectoryName;
	}

	public void setCompileLogsDirectoryName(String compileLogsDirectoryName) {
		this.compileLogsDirectoryName = compileLogsDirectoryName;
	}

	public String getTestManifestFileName() {
		return testManifestFileName;
	}

	public void setTestManifestFileName(String testManifestFileName) {
		this.testManifestFileName = testManifestFileName;
	}

	protected void getDropTokensFromList(String list) {
		StringTokenizer tokenizer = new StringTokenizer(list, ",");
		dropTokens = new Vector();
		for (; tokenizer.hasMoreTokens(); dropTokens.add(tokenizer.nextToken()))
			;
	}

	public String getDropTokenList() {
		return dropTokenList;
	}

	public void setDropTokenList(String dropTokenList) {
		this.dropTokenList = dropTokenList;
	}

	public boolean isBuildTested() {
		return isBuildTested;
	}

	public void setIsBuildTested(boolean isBuildTested) {
		this.isBuildTested = isBuildTested;
	}

	public boolean testsRan() {
		return testsRan;
	}

	public void setTestsRan(boolean b) {
		testsRan = b;
	}

	public Vector getDropTokens() {
		return dropTokens;
	}

	public void setDropTokens(Vector vector) {
		dropTokens = vector;
	}

	public String getTestResultsWithProblems() {
		return testResultsWithProblems;
	}

	public void setTestResultsWithProblems(String string) {
		testResultsWithProblems = string;
	}

	public String getBuildLabel() {
		return buildLabel;
	}

	public void setBuildLabel(String buildLabel) {
		this.buildLabel = buildLabel;
	}

	public String getHostname() {
		return hostname;
	}

	public void setHostname(String hostname) {
		this.hostname = hostname;
	}

	public String getPrefix() {
		return prefix;
	}

	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}

	public String getRecipients() {
		return recipients;
	}

	public void setRecipients(String recipients) {
		this.recipients = recipients;
	}

	public String getSender() {
		return sender;
	}

	public void setSender(String sender) {
		this.sender = sender;
	}

	public String getBuildLink() {
		return buildLink;
	}

	public void setBuildLink(String buildLink) {
		this.buildLink = buildLink;
	}

	static final String elementName = "testsuite";

	static final String testResultsToken = "%testresults%";

	static final String compileLogsToken = "%compilelogs%";

	public Vector dropTokens;

	public String testResultsWithProblems;

	private DocumentBuilder parser;

	public ErrorTracker anErrorTracker;

	public String testResultsTemplateString;

	public String dropTemplateString;

	private boolean testsRan;

	public String hostname;

	public String sender;

	public String prefix;

	public String recipients;

	public String buildLabel;

	public String buildLink;

	private boolean isBuildTested;

	public String dropTokenList;

	public String xmlDirectoryName;

	public String testResultsTemplateFileName;

	public String compileLogsDirectoryName;

	public String testManifestFileName;
}
